! --------------------------- BEGIN DENSITY ----------------------------
      function density(vels,
     :                 specify_dens_coeff,
     :                 dens_low, vel_low,
     :                 dens_high, vel_high, dd_dv, units_c)

! Assigns a density based on shear velocity.  If specify_dens_coeff = .true.,
! then a straight line is fit between the coefficients (and density = dens_low
! for vels < vel_low and density = dens_high for vels > vel_high).
! If specify_dens_coeff = .false, then a default equation is used,
! based on Boore (BSSA, 2016, pp. 316--320).

! Suggested values if specify coefficients (but using the builtin function
! is preferred, as is done if specify_dens_coeff = .false) (WARNING: these have NOT been 
! modified with consideration of the results in Boore, 2016):

!        dens_low = 2.0
!        vel_low = 0.1
!        dens_high = 2.7
!        vel_high = 5.0
!        dd_dv = 0.0

 
! Another relation to consider  density = 1.73 + 0.29* vels  
! (From fitting data in a model provided by H. Magistrale, probably based
! on a Nafe & Drake relation). (NOTE added 12 october 2010:  I found the data and a plot showing the
! regression equation.  It was dens = 1.742 + 0.2875*Vs; I redid the regression today
! and confirmed the values.  I do not know where I obtained the 1.73 value mentioned above).

! Dates: 12/05/95 - Written by D.M. Boore
!        12/08/95 - Added specify_dens_coeff and the coefficients
!        04/16/96 - Changed vel_low from 0.291 to 0.3
!        02/10/03 - changed slope_v to dd_dv and included it in parameter
!                   list (it was not saved before, which was an error).
!        03/01/07 - Use a different default relation between S-wave
!                   velocity and density (see my notes on the subject).
!                   Added character variable "units_c": kps, mps for 
!                   km/sec or m/sec.  The development of this relation is
!                   described in daves_notes_on_relating_density_to_velocity_v1.2.pdf,
!                   available from http://quake.wr.usgs.gov/~boore/daves_notes.php.
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        07/09/15 - Replace default relation with new one from daves_notes_on_relating_density_to_velocity_v2.0.pdf
!        11/08/15 - Replace default relation with new one from daves_notes_on_relating_density_to_velocity_v3.0.pdf
!        07/09/18 - The only change was that the default relation has now been published as 
!                   Boore (BSSA, 2016, pp. 316--320).  The equations are the same as for the 11/08/15 revision.
!        02/22/22 - Changed "(vel. ge. 0.30 .and. vel. lt. 3.55)" to "(vel .ge. 0.30 .and. vel .lt. 3.55)" (The previous version
!                   worked, but I do not like the space between "." and "lt" and "ge")

      character units_c*(*), units_uc*3
      logical specify_dens_coeff, firstcall
      data firstcall/.true./

!      if (firstcall) then
!        firstcall = .false.
!        if (.not. specify_dens_coeff) then
!          dens_low = 2.5
!          vel_low = 0.3
!          dens_high = 2.8
!          vel_high = 3.5
!        end if
!        dd_dv = (dens_high - dens_low)/(vel_high - vel_low)
!      end if

      call trim_c(units_c, nc_units_c)
      units_uc = ' '
      units_uc = units_c(1:3)
      call upstr(units_uc)
      

      if (.not. specify_dens_coeff) then
        if (units_uc(1:3) .eq. 'KPS') then
           vel = vels
        else if (units_uc(1:3) .eq. 'MPS') then
           vel = vels/1000.0
        else
           print *,' ERROR, units_uc = '//units_uc(1:3)//
     :             ' which is not a valid string; QUITTING'
           stop
        end if
        
        if (vel .lt. 0.30) then
          density = 1.0 + (1.53*vel**0.85)/(0.35+1.889*vel**1.7)
!          density = 1.0 + (1.22*vel)/(0.0948+vel)
        else
          vp = 0.9409 + 2.0947*vel -0.8206*vel**2 + 0.2683*vel**3
     :                                            - 0.0251*vel**4
          if (vel .ge. 0.30 .and. vel .lt. 3.55) then
            density = 1.74 * vp**0.25
          else
            density = 1.6612*vp - 0.4721*vp**2 + 0.0671*vp**3
     :                          - 0.0043*vp**4 + 0.000106*vp**5 
          end if
        end if
        
      else
        if (vels .lt. vel_low) then
          density = dens_low
        else if (vels .gt. vel_high) then
          density = dens_high
        else
          dd_dv = (dens_high - dens_low)/(vel_high - vel_low)
          density = dens_low + (vels - vel_low)*dd_dv
        end if
      end if

      return
      end
! --------------------------- END DENSITY ----------------------------
