      Program F4Nrattle

! Reads in an output file from Site_Amp and makes a file for use
! as input to Nrattle

! Note:  If the output is intended to be for a specific layered model,
! it is important NOT to specify frequencies when running site_amp, 
! because otherwise the layering will be for the depths corresponding to
! the frequencies rather than the model interfaces. For example, if 
! site_amp is being run only to create a model that will be used by
! F4Nrattl to create an input file for NRattle, then DO NOT SPECIFY 
! FREQUENCIES if the inout model is a specific constant-velocity layered
! model.  On the other hand,
! if site_amp is being used to make a layered model out of a model with
! gradients, then site_amp should be run with frequencies specified.

! Note:  To match the high-frequency decay produced by Q and overall kappa
! in site_amp, it is important to run nrattle with no attenuation in
! each layer (Q = 0.0 gives this, since Q < 1.0 is interpreted as 1/Q), 
! and then multiply amp by exp(-pi*kappa*f) in a separate step.  If the Q in 
! each layer is set to the overall travel time through the layers, divided by 
! the overall kappa, which seems reasonable, the high-frequency attenuation
! may be larger than desired because the waves may be reverberating in the
! layers and are thus spending more time in each layer than given by the
! single transit time across each layer (which is assumed in the kappa 
! calculation in site_amp).

!! Here follows a sample control file:
!!Control file for f4nrattle (data lines are free format)
!! (note that text lines preceded by "!" are ignored)
!!Revision of program involving a change in the control file on this date:
!   09/09/11
!!Name of site_amp file from which to obtain the velocity model for nrattle:
!! ofr_amp.asc
!  test.mdl.test.asc
!!Name of the nrattle control file to be created by f4nrattl:
!  nrattle.ctl
!!Prefix to use in constructing output file name
!!The output file names will be
!! <prefix>.<name of site_amp file>.complex_spectra.out 
!! <prefix>.<name of site_amp file>.amps4plot.out 
! test
!! Number of frequencies (including f = 0.0) and highest freq. (nfreq, fhigh):
!! If nfreq and fhigh are being specified for use in creating a time series via an FFT, 
!!         nfreq = npts/2 + 1
!!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)
!! where npts is the number of points in the FFT (a power of 2) 
!    400  50.0
!! Depth at which response is computed (0.0 for free surface):
!    0.0
!! Layer corresponding to halfspace (can be less than actual halfspace) 
!! (a large number means the source is in the halfspace) and 
!! angle of incidence (< 0.0 means obtain theta from site_amp file):
!    2000 30.0
!! Qinverse info (< 0.0 means obtain Qinverse from site_amp file):
!    -1.0

! Dates: 02/14/99 - Written by D.M. Boore, patterned after f4rattle.for
!        04/25/00 - Add guidelines regarding use to beginning
!        10/11/00 - Added sample control file to headers above
!        04/16/01 - Comment lines in nrattle control file have "!" before
!                   each, because nrattle now uses skipcmnt to skip the
!                   comment lines
!        02/10/03 - Use trim_c to strip leading and trailing blanks from
!                    file names (modern compilers allow blanks in file names,
!                    and this might produce errors if the file name obtained 
!                    from a control file is used as is--- for example, if
!                    the name in the control file is preceeded by blanks).
!        02/10/03 - Replace unformatted write with formatted write
!        08/19/05 - Renamed from "f4nrattl.for" to "f4nrattle.for"
!        08/20/05 - Replaced formatted read of nlay_src, theta_spcfy with
!                   "read(nu_ctl, *) nlay_src, theta_spcfy"
!        08/20/05 - Write out depth to bottom of each layer
!        01/12/06 - Change format to allow for larger halfspace velocity 
!                   (e.g., 3500 m)
!        10/17/10 - Allow for longer nrattle output file names
!        09/09/11 - Add a change date to the control file, and read the stem name for
!                   the two output files rather than the names of the two output
!                   files.
!        07/27/12 - Modification to account for small differences in site_amp output file
!                 - Modify for new order of nrattle.ctl input (halfspace info
!                   after the model).
!                 - Take into account the new output format of site_amp_batch 
!                   and site_amp (es11.5)
!        08/05/12 - Read site_amp input into dummy array so that I can use an unformatted
!                   read (so future revisions of site_amp that involve changes to
!                   output file column widths will not cause errors).
!        08/16/12 - Change "freq4halfspace" to "fmin4model", and change column 
!                   in f_samp containing the value of fmin4model
!        08/23/12 - Change column 
!                   in f_samp containing the value of fmin4model
!        03/08/13 - Increased dimensions of comments
!        12/20/15 - Add explanation of nfreq, fhigh needed for use in
!                   creating a time series using a FFT
!        07/18/17 - Corrected bug in reading site_amp file.  On 01 Jan16 I added slowness
!                   to the layer output but forgot to change f4nrattle accordingly.  
!                   Thanks to Hyun-Me Rhee for pointing out the error.
!        08/04/17 - Take new site_amp_batch output format into account (mainly headers
!                   have been modified).
!        05/08/21 - Replace write(*, '(a\)') with write(*, '(a)',advance='no')
!        09/03/21 - Because the qi info has been deleted from site_amp_batch output, remove
!                   the code here that read it when qi_spcfy < 0. Instead, set qi_samp = 0.0.
!                   If qi_spcfy >= 0 the value will be used, as before, in assigning a single value of
!                   damping to all layers of the model.
!        09/15/21 - Allow i5 for nout, rather than i4
!        10/06/21 - Fix bug when ndepths < nout.  For ease of programming, assume that the model information to
!                   be used starts on column 49.
!        03/18/22 - Qi is now written to the site_amp_batch output. 
!                 - Note: I found two dates for ctl files: 09/09/11 and 11/13/21.
!                   The only difference is in the text regarding naming the output files.
!                   I have updated the ctl file with the 11/13/21 text, but kept the 09/09/11 date.
!        04/06/23 - Correct "stem" name provided to nrattle to be that discussed in the ctl file (<prefix>.<name of site_amp file>)
 

      character stem_name*300, f_ctl*300, f_nr_ctl*300, 
     :          f_nr_out*300, f_nr_asc*300, 
     :          f_samp*300
      character comment(200)*100
      
!      real dum(11)
      real dum(6)
!      character cdum(????

      character date_ctl_correct*8, date_ctl_in*30
      
      logical f_file_exist

      f_file_exist = .false.
      do while (.not. f_file_exist)
        f_ctl = ' '
        write(*,'(a)',advance='no')
        
     :    ' Enter name of control file (cr=f4nrattle.ctl; '//
     :    'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl = 'f4nrattle.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_file_exist)
        if (.not. f_file_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

!Check version of control file
      date_ctl_correct = ' '
      date_ctl_correct = '09/09/11'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      call skipcmnt(nu_ctl, comment, ncomments)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      if (date_ctl_correct(1:nc_date_ctl_correct) /=
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; STOP!'
        close(nu_ctl)
        stop
      end if
      
! Read through the control file and process things along the way:

      call skipcmnt(nu_ctl, comment, ncomments)

      f_file_exist = .false.
      f_samp = ' '
      read(nu_ctl, '(a)') f_samp
      call trim_c(f_samp, nc_f_samp)
      inquire(file=f_samp(1:nc_f_samp), exist=f_file_exist)
      if (.not. f_file_exist) then
        write(*,'(a)') ' ******* FILE '//f_samp(1:nc_f_samp)//
     :        'DOES NOT EXIST ******* '
        write(*,'(a)') ' QUITTING!!!!'
        close(nu_ctl)
        stop
      end if
       
      call skipcmnt(nu_ctl, comment, ncomments)

      f_nr_ctl = ' '
      read(nu_ctl, '(a)') f_nr_ctl
      call trim_c(f_nr_ctl, nc_f_nr_ctl)
      call get_lun(nu_nr_ctl)
      open(unit=nu_nr_ctl, file=f_nr_ctl(1:nc_f_nr_ctl), 
     :     status='unknown')

      write(nu_nr_ctl,'(a)')  
     :   '!Control file for nrattle (data lines are free format)'
      write(nu_nr_ctl,'(a)')  
     :   '! (note that text lines preceded by "!" are ignored)'
      write(nu_nr_ctl,'(a)')  
     :   '!Revision of program involving a change in '//
     :      'the control file on this date:'
      write(nu_nr_ctl,'(a)')  
     :   '    09/09/11'
     
      write(nu_nr_ctl,'(a)') 
     :   '!Stem name for output files (the file containing model '//
     :           'information and complex spectra will be named' 
      write(nu_nr_ctl,'(a)') 
     :   '! <stem>.nrattle_complex_spectra.out' 
      write(nu_nr_ctl,'(a)') 
     :   '! and the one containing the absolute spectral amplitudes '//
     :           'used in plotting will be named' 
      write(nu_nr_ctl,'(a)') 
     :   '! <stem>.nrattle_amps4plot.out'
      call skipcmnt(nu_ctl, comment, ncomments)
      stem_name = ' '
      read(nu_ctl, '(a)') stem_name
      call trim_c(stem_name,nc_stem_name)
      write(nu_nr_ctl,'(2x,a)') 
     :   stem_name(1:nc_stem_name)//'.'//f_samp(1:nc_f_samp)

      write(nu_nr_ctl,'(a)') 
     :   '!Number of frequencies (including f = 0.0) and highest '//
     :               'freq. (nfreq, fhigh):'   
      write(nu_nr_ctl,'(a)')
     :   '! If nfreq and fhigh are being specified for use in '//
     :     'creating a time series via an FFT,' 
      write(nu_nr_ctl,'(a)')
     :   '!         nfreq = npts/2 + 1'
      write(nu_nr_ctl,'(a)')
     :   '!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)'
      write(nu_nr_ctl,'(a)')
     :   '! where npts is the number of points in the FFT'//
     :    ' (a power of 2)' 
      call skipcmnt(nu_ctl, comment, ncomments)
      read(nu_ctl,*) nfreq, fhigh
      write (nu_nr_ctl,'(1x,i5,1x,f8.2)') nfreq, fhigh

      write(nu_nr_ctl,'(a)') 
     :   '!Depth at which response is computed (0.0 for free surface):'                   
      call skipcmnt(nu_ctl, comment, ncomments)
      read(nu_ctl,*) zdepth
      write (nu_nr_ctl,'(1x,f7.2)') zdepth

!%%%%%%%%%%%%%%%%%%%%%%%%%%

      call skipcmnt(nu_ctl, comment, ncomments)
      read(nu_ctl, *) nlay_src, theta_spcfy
      
!DEBUG
      print *,' nlay_src, theta_spcfy = ',
     :          nlay_src, theta_spcfy
!DEBUG

! Q info:
      call skipcmnt(nu_ctl, comment, ncomments)
      read(nu_ctl, *) qi_spcfy
      if (qi_spcfy >= 0.0) then
        if (qi_spcfy > 1.0) qi_spcfy = 1.0/qi_spcfy   ! convert q to 1/q
      end if

!DEBUG
      print *,' qi_spcfy = ',
     :          qi_spcfy
!DEBUG
      
      close(nu_ctl)

! Extract parameters from the site_amp output file and write to f4nrattl output:

      call get_lun(nu_samp)
      open(unit=nu_samp, file=f_samp, status='unknown')

      call skip(nu_samp,3)
      read(nu_samp, '(t23,e10.3, t34,e10.3, t81,f6.1)') 
     :  velsource, denssource, theta
      print *, ' velsource, denssource, theta = ', 
     :           velsource, denssource, theta
      call skip(nu_samp,1)
      read(nu_samp,'(t13,i5, t33,i5, t146,es10.3)') 
     :              ndepths, nout, fmin4model
      call skip(nu_samp,2)  ! skip "Velocity units" and column headers

      write(nu_nr_ctl,'(a)') 
     :   '!Model information:' 
      write(nu_nr_ctl,'(a)')  
     :   '!  Nlayers above the actual halfspace; fmin4model:'
      write(nu_nr_ctl,'(a, 1x,i5, 1x,es9.3)') '!', nout, fmin4model
      write(nu_nr_ctl,'(a)')   
     : '!For each layer- thickness, Vs, density, 1/Q, depth to bottom'
      write(nu_nr_ctl,'(a)') 
     : '! (if 1/Q > 1.0, the program assumes that the value '//
     : 'represents Q; note that damping in percent = 100*0.5*1/Q):'

      d2bot = 0.0
      do i = 1, nout
        read(nu_samp,'(72x,5(1x,es11.5))')
     :              thick, vel, dens, qi_samp, slowness 
!        ndum = 11     ! 2021-09-03
!        ndum = 10
!        read(nu_samp,) (dum(j), j=1, ndum)
!        thick = dum(7)
!        vel = dum(8)
!        dens = dum(9)
!        slowness = dum(10)
!!        qi_samp = dum(11)
!        qi_samp = 0.0  ! 2021-09-03
       
        if (qi_spcfy >= 0.0) then
          qi_out = qi_spcfy
        else
          qi_out = qi_samp
        end if
        d2bot = d2bot + thick
        write(nu_nr_ctl, '(1x,i4,5(1x,es11.5))') 
     :     i, thick, vel, dens, qi_out, d2bot
      end do

      close(unit=nu_samp)

      write(nu_nr_ctl,'(a,2x,a)') '!',   
     :  'Halfspace Vs and density (1/Q is automatically set to 0.0):'
      write(nu_nr_ctl, '(1x,f7.2,1x,f5.2)') velsource, denssource

! Nlay_src and angle of incidence (theta) info:
      write(nu_nr_ctl,'(a)') 
     :   '!Layer corresponding to halfspace (can be less than '//
     :          'actual halfspace)'          
      write(nu_nr_ctl,'(a)') 
     :   '! (a large number means the source is in the halfspace) '//
     :           'and'                     
      write(nu_nr_ctl,'(a)') 
     :   '! angle of incidence (< 0.0 means obtain theta from '//
     :           'site_amp file):'             
      if (nlay_src > nout + 1) nlay_src = nout + 1
      if (theta_spcfy > 0.0) theta = theta_spcfy
      write(nu_nr_ctl,'(1x,i5,1x,f6.2)') nlay_src, theta

! All finished:
      
      close(unit=nu_nr_ctl)

      stop
      end

      include 'site_amp_util_subs.for'