      PROGRAM nRattleBatch

!  Haskell-Thomson transfer function (SH) for horizontally stratified medium.
!  Calculate complex spectrum from DC to Nyquist.

! This version is based on site_amp_batch. It computes the amplifications for a series of velocity
! model files, and therefore does not require site_amp_batch to be run, followed by f4nrattle, and then
! followed by nrattle.

! This version creates only the equivalent of the nrattle "nrattle_amps4plot.out" file
! and not the "nrattle_complex_spectra.out" file.  If that file is desired, then the site_amp_batch, f4nrattle, nrattle
! series of programs needs to be run.

! Q or 1/Q can be specified, and the cumulative kappa through the velocity
! model will be computed (starting from the top).  This value of kappa or a 
! specified kappa can be used to compute the diminution due to kappa.

! Density can be specified in three ways:
!
! 1) read from model file, with values of 0.0 replaced using a density--Vs relation
! 2) Computed from Vs using a built-in density--Vs relation (in site_amp, it was
!    possible to specify coefficients of this relation, but I have not allowed that
!    option here).
! 3) Reading in dens_default and dens_ref (the simplest case, allowing density
!    variations to have minimal or no impact).

! The transfer function is normalized by twice the amplitude of the incoming
! wave so that the amplitude of the transfer function approaches
! unity for low frequencies.   This is equivalent to the transfer function
! being the ratio of site response at the surface above the soil layer and
! the site response at the surface of a "rock" site that is a uniform material
! with velocity and density equal to those in the halfspace below the
! layers.  If the complex transfer function is to be used to
! deconvolve surface motions to obtain the input motion below the layers, the 
! transfer function should first be multiplied by 2.0 before dividing the 
! spectrum of the surface motion by the complex transfer function.  If that 
! input motion is then to be convolved with the transfer function for a 
! different site, the factor of two has to be accounted for again.  The result
! is that the factors of two cancel, so that the process of deconvolution and
! convolution is given simply by multiplying the spectrum of the observed
! motion at site 1 (S_1) by the ratio of the complex transfer functions
! at sites 1 and 2: S_2 = S_1 * (T_2/T_1)

!
!  Haskell's approach: Propagate particle velocity and stress through
!    (m)th layer [from (m-1)st boundary to (m)th boundary].
!  This approach: Propagate coefficients of up- and downgoing waves
!    across (m)th boundary [from (m+1)st layer to (m)th layer].
!  Mixed units are OK. Everything cancels.
!
!  C. Mueller redid this from scratch - not like Joyner's anymore. (7/84, CSM).
!  FFT sign conventions are Aki-Richards (Box 5.2).


! Sample input (from file nRattleBatch.ctl) (the line below is line 1 of the input):
!!Control file for nRattleBatch
!! Revision of program involving a change in the control file on this date:
!   02/06/24
!!Name of summary file:
!   nRattleBatch.sum
!!Write file with velocity model and complex amplifications (in addition to the file with cabs(ComplexAmps)
!!  intended for plotting)?
! Y
!!Write file with SRI amps?
! Y
!!Are the velocities from a suspension log survey?
!!If yes, assume that the velocity value is at the midpoint of the layer
!   N
!!Velocity for first layer (0 = use the first velocity of the input)
!! Use this for sensitivity studies of vavg for PS log models that do not
!! extend to the surface.   
!   0.0
!!Depth to bottom (d2b) or layer thickness (thk) (note: a value of 0.0 for
!!the first depth from the model file will signal that the model is a
!!piecewise continuous model)?
!   d2b
!!Lines to skip to reach data (nskip):
!! nskip<0: use skipcmnt to skip a block of header lines starting with the 
!!          exclamation character ("!"), followed by skipping abs(nskip)-1 
!!          header lines.  This option is included so that the program can compute
!!          the amplifications for a group of model files that have a variable
!!          number of headers startng with "!" before a line with column headings.
!! nskip=0: skip no lines
!! nskip>0: skip the number of lines specified by nskip
!!
!! -1 ! the data are preceeded by one or more lines starting with "!"
!! -2 ! the data are preceeded by one or more lines starting with "!", followed 
!!      by a single line with column headers.
!   1 ! skip one line to reach the lines of model data
!!
!!Use density in model file?
!   N
!!dens_default (if layer density = 0.0, either because the densities from the 
!!  input file are to be used, but these are 0.0, or because the density is not
!!  to be obtained from the input file, then use this as the layer density unless it
!!  is 0.0, in which case obtain the density from my density--velocity
!!  subroutine, using the equations in Boore (BSSA, 2016, pp. 316--320)):
!   0.0
!!Use Q from model file (note that Q < 1.0 is taken is 1/Q)?
!   N
!! kappa for computation (enter 0.0 for no kappa computation):
!   0.0
!!Ncolumns to read, column numbers for depth or thickness (allows for the first column
!!being an index),velocity, density, and Q (note that if density and/or Q are not read
!!from the file, the column numbers for these variables will be ignored.  But dummy
!!values need to be included in the control file).
!! WARNING: Be careful to maintain consistency between whether density and/or Q
!! are read from input file and the entries below.  For example, suppose the input file
!! contains thickness, vs, density, and Q.  Then the following table illustrates the
!! proper input for Ncolumns, etc:
!** IMPORTANT **  Ncolumns to read should be consistent with the number of model columns actually present, although
! placeholders are needed so that there are 5 entries in the line below (see examples in the table below).
! Density from input  Q from input  Ncolumns, etc
!          N               N        2 1 2 2 2
!          Y               N        3 1 2 3 3
!          Y               Y        4 1 2 3 4
!          N               Y        4 1 2 3 4 (I'm not sure that this works)
!! If specified incorrectly, the program will probably work, but the velocity
!! model will be incorrect.  ALWAYS CHECK THE VELOCITY MODEL IN THE OUTPUT FILE!
!   2 1 2 3 4
!!Enter units of velocity (kps, mps) (depth units are assumed to be consistent
!!with the velocity units---km for kps and m for mps):
!   kps
!!
!!Parameters controlling creation of constant-velocity layered (CVL) model to be used in the nrattle calculations
!!  
!!Enter freqlow4model, freqhigh4model, nfreq4model for creation of CVL model.
!!  if nfreq4model = 0: the CVL model will be based on the depths to the interfaces of the input model 
!!                      (even if the input model is made up of linear segments; if it is a CVL model, the model used for the amp calcs will be
!!                      the same as the input model).
!!     nfreq4model < 0: the greatest depth for the CVL model will correspond to the bottom of the input model, with 
!!                      abs(nfreq4model) layers whose boundaries are determined by a range of frequencies from
!!                      the QWL frequency corresponding to the bottom of the input model (this replaces freqlow4model) to 
!!                      fhigh4model.
!!     nfreq4model > 0: the CVL model will have nfreq4model layers whose boundaries are determined by the range of QWL depths corresponding to flow4model 
!!                      to fhigh4model. Note that it is possible for the deepest depth to be shallower or deeper than the bottom of the input model,
!!                      depending on the comparison of flow4model to the QWL frequency for the bottom of the input model.  If deeper, the model is
!!                      essentially extrapolated as a halfspace with velocity and density corresponding to the bottom of the input model.
!0.05 100.0 -1200
!!Log-spaced frequencies for model (probably should be "Y", based on sample runs I've made)?
!   Y
!!
!!Enter freqlow, freqhigh, nfreq for amplification calculations
!! If nfreq and fhigh are being specified for use in creating a time series via an FFT,
!!         nfreq = npts/2 + 1
!!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)
!!         flow = 0.0
!! where npts is the number of points in the FFT (a power of 2)
!   0.1 100.0 1200
!!Log-spaced frequencies for amplifications?
!   Y
!!v_ref, dens_ref for amp calculation (dens_ref = 0.0 means that it will be obtained from
!! my relation between vel and dens; if dens_ref /= 0.0, then it will be used even if
!! "Use density from model file" is N):
!   3.5 2.72
!!Angle of incidence at source level (in degrees):
!   00.0
!!Depth at which response is computed (0.0 for free surface):
!   0.00
!! Layer corresponding to halfspace (can be less than actual halfspace) 
!! (a large number means the source is in the halfspace):
!   9000
!Character string to append to end of each input file to create
!the name of the output file (an additional string will be added automatically to make the
!output file name more descriptive, as there can be up to three output files, and to end in ".out"):
!   .1200logfreqsToModelBottomto100
!!List of input files
!!Make sure that there are no blank entries!
!!The format of the input are consecutive columns containing the 
!!depth/thickess and velocity, after header lines starting with "!"
!   bj97_generic_rock_from_interpolate_xls.mdl
!   laa_cy14_extrapolated.equal_tt_layers.D2bVsInput.mdl
! stop
  
! The line above is the last line of the input (without the comment character "!")

! Here is a sample input file for the velocity model(column 1 of the input starts after the "*"
! character):
! Sample input file to site_amp; not for a specific application
!       Depth    SVel     Dens   1/Q
!           0   0.300      2.0   0.10
!       0.001   0.300      2.0   0.10  ! constant from 0.000 to 0.001
!       0.005   0.500      0.0   0.06  ! linear gradient from 0.001 to 0.005
!       0.020   0.800      0.0   0.05  ! linear gradient from 0.005 to 0.020
!       0.040   1.000      0.0   0.04  ! linear gradient from 0.020 to 0.040
!       0.040   1.500      0.0   0.04  ! step increase at 0.040
!       0.100   1.500      2.5   0.02  ! constant vel from 0.040 to 0.100
!       0.300   2.000      0.0   0.02  ! linear gradient from 0.100 to 0.300
!       0.300   2.500      0.0   0.02  ! step increase at 0.300
!       0.500   2.500      2.6   0.01  ! constant from 0.300 to 1.000,
!       1.000   2.500      2.6   0.001 !   but density and 1/Q differ

 
! If the layer is to be in the halfspace, then the layer number corresponding
! to the halfspace can be set to be a large integer; the program replaces it 
! with the proper value.  The only critical integer is the number of layers 
! above the real halfspace.

! The program as written by C. Mueller specified input parameters appropriate
! for using the response in a separate program that multiplied the complex
! response by the FFT of the input time series to obtain an output time
! series.  This required specifying nfreq and fhigh compatible with
! the FFT.  He found it most convenient to specify parameters SAMP and MX for
! this.  These are related to nfreq and fhigh as follows:

!         mx  = 1 + ln(nfreq -1)/ln(2)
!         samp = 2*fhigh
!

! If nfreq and fhigh are being specified for use in creating a time series via an FFT, 
!         nfreq = npts/2 + 1
!         fhigh = 1.0/(2.0*dt) (the Nyquist frequency)
! where npts is the number of points in the FFT (a power of 2) 

! The original version contained the following code to convert the
! input parameters MX and SAMP into the number of frequency points (INYQ, where
! this is the number of evenly spaced samples between f = 0.0 and the Nyquist
! frequency), the sample interval (DT), the length of the FFT time series (T),
! and the frequency spacing (DELF).

!      DT = 1.0/SAMP
!      IPW2 = 2**MX !Number of points in corresponding time series.
!      INYQ = IPW2/2+1
!      T = IPW2/SAMP
!      DELF = 1.0/T

!
!  Written by Chuck Mueller, USGS.
!    Changed input to skip comment lines at top of input file, 8/86.
! Unless indicated, all mods below by D.M. Boore
! Dates: 12/18/95 - Dave Boore modified the program for the pc and increased
!                   array dimensions, used list-directed input
!        03/12/96 - i/o changes, asking for file names and writing a file
!                   with fewer header lines for use in COPLOT (to be imported
!                   as a wrapped ASCII file).  Also added file stem name
!                   to output column.
!        01/28/98 - Read nlay_src from the last line of the input.  Use this
!                   to determine of the effective halfspace should be above 
!                   the actual halfspace of the model.  In this way, can run
!                   rattle with various source depths without needing to
!                   manually edit the input file.  Note that if the source
!                   layer is to the real halfspace, then set 
!                   nlay_src = nlay + 1
!        02/24/98 - Suggest rattle.asc for file to be imported into CoPlot, 
!                   rather than rattle.4cp.  Also, include relation between
!                   sps, mx and fhigh, nfreq (f4rattle now asks for the more
!                   natural fhigh, nfreq, and computes sps and mx.  Rattle
!                   still expects the latter as input).
!-----
!        07/27/98 - Reformulated by R. Herrmann to
!                   a) use a stabler formulation for surface motion that
!                      is less susceptible to numerical instabilities for
!                      low Q media
!                   b) Did not use Aki + Richards definition of 
!                      Fourier Transform
!                      Instead
!                      G(f) = int from { - inf } to { inf } 
!                            g(t) exp ( -j 2 pi f t ) dt
!                      g(t) = int from { - inf } to { inf } 
!                            G(f) exp (  j 2 pi f t ) df
!                      The controls the definition of upward and downward
!                      wavefields and the sign for the affect of 
!                      Q on complex velocity
!-----
!        10/31/98 - Read in 1/Q rather than Q (convert input to 1/Q
!                   if the input value is greater than 1.0)
!        11/05/98 - Suggest *.out and *.asc output file names
!        01/11/99 - Complete revamping of input parameters (see text above for
!                   new format and parameters)
!        10/11/00 - Corrected error in sample control file above
!        12/22/00 - Clarified choice of nfreq, fhigh for use in FFT
!        04/16/01 - Allow for variable number of comment lines in control file
!                   by using subroutine skipcmnt.  If there is more than one
!                   comment lines, then each line must be preceded by "|".
!        04/16/01 - Bring in subroutines via include nrtlsubs.for, where
!                   nrtlsubs.for is made each time the program is compiled
!                   by calling clnrattl.bat.  This guarantees that the latest
!                   versions of the subroutines are used.
!        05/01/01 - If effective half space is less than the actual half space,
!                   the old version still used the velocity and density given
!                   in the last line of the control file.  I (DMB) has changed
!                   this so that betan and rhon are given by the values in the
!                   layer that is the effective half space. 
!        02/10/03 - Use trim_c to strip leading and trailing blanks from
!                   file names (modern compilers allow blanks in file names,
!                   and this might produce errors if the file name obtained 
!                   from a control file is used as is--- for example, if
!                   the name in the control file is preceeded by blanks).
!        02/10/03 - Commented out computation of sumt, sumtq; those 
!                   quantities are not part of the nrattle standard output.
!        08/19/05 - Print name of output file to screen.
!        01/24/08 - Modernize by using get_lun, get subroutines from nrtlsubs.for
!                   (this is created by clnrttl.bat)
!        04/06/10 - Added period to output for plotting.
!        05/12/10 - Add depth to bottom of each layer to output file containing
!                   the model (nu_cabsAmps).
!                 - Call system "date_and_time" routine rather than the compiler-dependent
!                   "datetime.for"
!        05/14/10 - Remove units of velocity, thicknesses from output files
!                   (the units can be mks or cgs or anything else as long as they
!                   are consistent).
!        10/17/10 - Allow for longer output filenames
!        09/09/11 - Add a change date to the control file, and read the stem name for
!                   the two output files rather than the names of the two output
!                   files.
!        09/11/11 - Add name of output file to output
!        09/13/11 - Delete specification of number of model layers in control file,
!                   and put the layer number for the effective halfspace at the bottom
!                   of the control file.
!        03/02/12 - Increase precision of frequency output, and write as fixed format.
!        08/18/12 - Allow more layers in model
!        08/20/12 - Allow more layers in model
!        03/08/13 - Increased dimension of comment
!        12/20/15 - Write nlayrs before model, so know how many lines to skip to get to the
!                   output when reading the file in another program.
!                 - Change spacing of some of the output lines.
!        11/29/21 - Added new sample nrattle.ctl lines above, to make it consistent with the current version
!                   of f4nrattle output
!        02/02/24 - Put the amp computations into a subroutine
!        02/06/24 - This version was created by D. Boore, using code from site_amp_batch to create a constant-velocity layered
!                   model and code from nrattle to compute full-resonance amplifications for this model. It computes the 
!                   amplifications for a series of velocity model files, and therefore CVL model
!        02/10/24 - Add possibility of extrapolating the model to depth=zx, vel=vx, with nxtrap log z points, using either a linear (xtraptype='L')
!                   or power law function (xtraptype='P').
!                   fmin is returned both to the bottom of the input model and the final, output model. This can be used in normalizing frequency
!                   in determining eta for SRImod.
!                 - Major restructuring, putting the code after a piecewise continuous model has been developed into the 
!                   construct_constant_velocity_model subroutine (requiring a major change in that subroutine).  Doing this allows
!                   for extrapolating the input model, if desired, within the main program, and then calling construct_constant_velocity_model
!                   with that model.  I do need to compute and save fmin4model_in and fmin4model_final, as those frequencies will be used in constructing
!                   normalized frequencies for use in a later program that computes FR, SRI, and eta in one program.
!        02/20/24 - Add fmin4OrigModel and fmin4XtrapModel to output of cabs(Amp)
!                 - Add option for writing SRI amps
!        02/21/24 - Change output file names to be more descriptive and always end in "out"
!                 - Include SRI amps using the modified eta, if option to compute SRI amps is chosen
!        02/24/24 - Removed ndepthsAfterAnyExtrapolation and fmin4XtrapModel variable. It was there in case an extrapolation 
!                   option of the original model was an option, but I decided against including that option.
 


      implicit none

      integer, parameter :: MLAYR = 10000
      integer, parameter :: MNFREQS =4000
      
      integer icol_q,nc_units_c,nfreq4model,nfreq,nlay_src,nc_ext,
     :   nc_f_mdl,nc_f_cabsAmps,nu_cabsAmps,nc_f_MdlAmps,nu_MdlAmps,
     :   nc_f_SriAmps,nu_SriAmps,
     :   nu_mdl,nskip_more,nentries,i,nlayers,idepth,ilayer,ndepths,
     :   ndepthsOriginalModel,
     :   nCVLlayers,nfreq_in,IFRQ,nchar_tb

      integer nc_f_ctl,nu_ctl,nc_cmnts2skip,nc_version_in,
     :  nc_version_ctl,nc_f_sum,nu_sum,nc_buf,nc_type_of_depth,
     :  nskip,icol_dens,icol_vel,icol_dh,ncol

      real freqhigh4model,freqlow4model,freqhigh,freqlow,dens_ref_in,
     :   vel_ref,aoi_src,p,zdepth,dd_dv,vel_high,dens_high,vel_low,
     :   dens_low,dens_ref,fmin4OrigModel,
     :   akappa4SRIamps,akappa_SRIamps(MNFREQS),amp_SRIamps(MNFREQS),
     :   freq_SRIamps(MNFREQS),f,per

      real pi,d2r,vel1,dens_default,akappa_fixed,density

      real Z(MLAYR), H(MLAYR), vs(MLAYR), rho(MLAYR), qis(MLAYR)
      real freq(MNFREQS)
      complex CmplxAmp(MNFREQS)
      character date_c*8, time_c*10

      real depth(MLAYR), vel(MLAYR), dens(MLAYR), qi(MLAYR),
     :     akappa(MLAYR), 
     :     d2b(0:MLAYR), 
     :     depth_in(MLAYR), vel_in(MLAYR), dens_in(MLAYR), 
     :     q_in(MLAYR), del_z(MLAYR), dens_int(MLAYR)

      real 
     :     freq_cabsAmps(MNFREQS), 
     :     amp_cabsAmps(MLAYR), akappa_cabsAmps(MLAYR), 
     :     ampxkappa_cabsAmps(MLAYR) 

      real thick_layer(MLAYR), vel_layer(MLAYR), dens_layer(MLAYR),
     :   qi_kappa_fixed(MLAYR), qi_layer(MLAYR), 
     :   d2lyrbot(0:MLAYR), tt2lyrbot(0:MLAYR), 
     :   avgdens2lyrbot(0:MLAYR), akappa2lyrbot(0:MLAYR), 
     :   avgvel2lyrbot(MLAYR), avgaoi2lyrbot(MLAYR), 
     :   ampxkappa_SRIamps(MNFREQS),
     :   dum1, dum2, fnorm(MNFREQS), log10fnorm(MNFREQS), eta(MNFREQS),
     :   EtaFromCSV, EtaForSRImodFromCSV


      real dum(10)
      
      real avgvel(MLAYR), avgdens(MLAYR), avgaoi(MLAYR)

      integer iout(MLAYR), status, nc_f4eta, nu_f4eta, i4eta, neta
      
      character f_ctl*120, f_sum*120, f_mdl*120, f_cabsAmps*120,
     :          constant_vel*1, dtbot*1, ext*100,
     :          buf*120, buf4*4,
     :          answer*1, f_MdlAmps*120, f_SriAmps*120,
     :          bffr*80, rho_col_exist*3, use_rho_col*3
      character tempbuf*8, col1*12, col2*12, col3*12, 
     :          col4*12, col5*12, col6*12
      character col_d*10, col_v*10, col_r*10, col_qi*10, 
     :          col_f*10, col_a*10, col_k*10, col_ak*10,
     :          cbuf(28)*11, spcfy_freq*1, units_c*10,
     :          col_z_other_units*11, col_v_other_units*11
       character cmnts2skip(200)*100, type_of_depth*10
 
       character version_ctl*8, version_in*30, f4eta*200
 
      logical susp_log, dens_from_model, q_from_model,
     :        f_mdl_exist, f_exist, specify_freqs,
     :        log_spaced_freqs, log_spaced_freqs4model, 
     :        computeSRImodFromMeanEta
      logical specify_dens_coeff, writeFileWithModel, writeSRIamps


      pi = 4.0 * atan(1.0)
      d2r = pi/180.0
 
!**********
!
!  GET INPUT
!
!**********

      call date_and_time(date_c,time_c)



!Get name of control file and open the file:

      write(*,*)

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :      '   Enter name of control file (cr=nRattleBatch.ctl; '//
     :         'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl='nRattleBatch.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl, file=f_ctl(1:nc_f_ctl), status='unknown')

!Check version of control file
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      version_in = ' '
      read(nu_ctl,'(a)') version_in
      call trim_c(version_in,nc_version_in)
      
      version_ctl = ' '
      version_ctl = '02/06/24'
      call trim_c(version_ctl,nc_version_ctl)

      if (version_ctl(1:nc_version_ctl) /= 
     :    version_in(1:nc_version_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        write(*,'(a)') 
     :     ' The correct date is '//version_ctl(1:nc_version_ctl)
        close(nu_ctl)
        stop
      end if
      
 !Get summary file name (NOTE: Nothing is written to this file at present, but it might be in a later version)
       call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
       f_sum = ' '
       read(nu_ctl,'(a)') f_sum
       call trim_c(f_sum, nc_f_sum)
       call get_lun(nu_sum)
       open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
 
!Write file with velocity model and complex amplifications?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         writeFileWithModel = .true.
      else
         writeFileWithModel = .false.
      end if

!Write file with SRI amplifications?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         writeSRIamps = .true.
      else
         writeSRIamps = .false.
      end if

!Get name of csv file with eta for SRImod (placeholder read, even if not used)
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f4eta = ' '
      read(nu_ctl,'(a)') f4eta
      call trim_c(f4eta, nc_f4eta)

      if(writeSRIamps) then
        call get_lun(nu_f4eta)
	open(unit=nu_f4eta, file=f4eta(1:nc_f4eta), status='unknown')
        call skip(nu_f4eta, 1)
        i4eta = 0
        read eta csv file: DO
          read(nu_f4eta,*,iostat=status) dum1, dum2
          if(status /= 0) then
            EXIT read eta csv file
          end if
          i4eta = i4eta + 1
          fnorm(i4eta) = dum1
          log10fnorm(i4eta) = log10(dum1)
          eta(i4eta) = dum2
!DEBUG
        write(*,'(1x,a, 1x,i5, 2(1x,es10.3))') 'i4eta, fnorm, eta: ', 
     :                       i4eta, dum1, dum2
!DEBUG

        END DO read eta csv file
        neta = i4eta
      end if

!Are velocities from a suspension log?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         susp_log = .true.
      else
         susp_log = .false.
      end if

!Get velocity of first layer:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vel1
      
!Depth to bottom (d2b) or thickness (thk)?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      type_of_depth = ' '
      read(nu_ctl, '(a)') type_of_depth
      call trim_c(type_of_depth, nc_type_of_depth)
      call upstr(type_of_depth)

!Lines to skip (<0 means use skipcmnt; 0 means skip none):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) nskip
!!DEBUG
!      write (*,'(1x,a,1x,i3)') ' nskip = ', nskip
!!DEBUG

!Get density from the model file?:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         dens_from_model = .true.
      else
         dens_from_model = .false.
      end if
      
      specify_dens_coeff = .false.  ! The use of this logical variable 
                                    ! is a carryover from earlier versions;
                                    ! it is needed by the subroutine density() 

!Get dens_default (see comments in control file; set to 0.0 if want the density
! to be obtained from my density--velocity subroutine):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) dens_default 
      
!Get Q from the model file?:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         q_from_model = .true.
      else
         q_from_model = .false.
      end if

!Specify kappa:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) akappa_fixed

!Ncolumns, Column numbers for depth/thickness, velocity, density, Q:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) ncol, icol_dh, icol_vel, icol_dens, icol_q

!Specify units of velocity (kps, mps):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      units_c = ' '
      read(nu_ctl,'(a)') units_c
      call trim_c(units_c, nc_units_c)
      call upstr(units_c)
      if (units_c(1:3) /= 'MPS' .and. units_c(1:3) /= 'KPS') then
        print *, ' ERROR: units_c(1:3) >= MPS or KPS; it equals '//
     :           units_c(1:3)//' STOP!!!'
        stop
      end if
      
!Enter freqlow4model, freqhigh4model, nfreq4model for creation of CVL model.
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) freqlow4model, freqhigh4model, nfreq4model
 
!log-spaced frequencies for model?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         log_spaced_freqs4model = .true.
      else
         log_spaced_freqs4model = .false.
      end if
 

!Enter freqlow, freqhigh, nfreq for amplification calculations 
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) freqlow, freqhigh, nfreq_in
      
!log-spaced frequencies?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         log_spaced_freqs = .true.
      else
         log_spaced_freqs = .false.
      end if
 
      call fill_freq_array(freq, nfreq_in, freqlow, freqhigh, 
     :                        log_spaced_freqs)

!Get reference velocity and density (set to 0.0 if want the densities to be obtained
! from my density--velocity subroutine):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vel_ref, dens_ref_in
      
!Enter angle-of-incidence at source level (in degrees): '
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) aoi_src
      p = sin(d2r*aoi_src)/vel_ref  ! ray parameter
      
!Depth at which response is computed (0.0 for free surface):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) zdepth
      
! Layer corresponding to halfspace (can be less than actual halfspace) 
! (a large number means the source is in the halfspace):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) nlay_src
 
!Character string to append to end of each input file to create
!the name of the output file (be sure to include the period if wanted):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      ext = ' '
      read(nu_ctl, '(a)') ext
      call trim_c(ext,nc_ext)
 
!**********
!
!  READ AND PROCESS VELOCITY MODELS
!
!**********

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

!!DEBUG
!      call get_lun(nu_debug)
!      open(unit=nu_debug,file='debug.out',status='unknown')
!!DEBUG
      

      loop over files: DO

        buf = ' '
        read(nu_ctl,'(a)',iostat=status) buf
        if(status /= 0) then
          EXIT loop over files
        end if
        call trim_c(buf, nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(nc_buf == 0 .or. buf(1:1) == '!') then
          CYCLE loop over files
        end if
        if(buf4(1:4) == 'STOP') then
          EXIT loop over files
        end if
      
        f_mdl = ' '
        f_mdl = buf(1:nc_buf)
        call trim_c(f_mdl, nc_f_mdl)
        f_exist = .false.
        inquire(file=f_mdl(1:nc_f_mdl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') 
     :  ' ******* FILE '//f_mdl(1:nc_f_mdl)//' DOES NOT EXIST ******* '
          write(*,'(a)') ' Skipping to next file'
          CYCLE
        end if
 
!########################################## 
!Get name of output files and open the file:

        f_cabsAmps = ' '
        f_cabsAmps = f_mdl(1:nc_f_mdl)//ext(1:nc_ext)//'.FrAmps.out'
        call trim_c(f_cabsAmps, nc_f_cabsAmps)
        call get_lun(nu_cabsAmps)
        open(unit=nu_cabsAmps, file=f_cabsAmps(1:nc_f_cabsAmps), 
     :       status='unknown')

        if(writeFileWithModel) then
        
          f_MdlAmps = ' '
          f_MdlAmps = f_mdl(1:nc_f_mdl)//ext(1:nc_ext)//
     :                '.FrModelAndAmps.out'
          call trim_c(f_MdlAmps, nc_f_MdlAmps)
        
          call get_lun(nu_MdlAmps)
          open(unit=nu_MdlAmps, file=f_MdlAmps(1:nc_f_MdlAmps), 
     :         status='unknown')
        
        end if

        if(writeSRIamps) then
        
          f_SriAmps = ' '
          f_SriAmps = f_mdl(1:nc_f_mdl)//ext(1:nc_ext)//'.SriAmps.out'
          call trim_c(f_SriAmps, nc_f_SriAmps)
        
          call get_lun(nu_SriAmps)
          open(unit=nu_SriAmps, file=f_SriAmps(1:nc_f_SriAmps), 
     :         status='unknown')
        
        end if

! Open model file:
        write(*,'(a)') ' Processing velocity model file: '//
     :                      f_mdl(1:nc_f_mdl)
         
        call get_lun(nu_mdl)
        open(nu_mdl,file=f_mdl(1:nc_f_mdl),status='unknown')
        
        if (nskip < 0) then
          call skipcmnt(nu_mdl, cmnts2skip, nc_cmnts2skip)
          nskip_more = abs(nskip) - 1
          call skip(nu_mdl, nskip_more)  
        else if (nskip > 0) then
          call skip(nu_mdl, nskip)
        end if
        
        
        nentries = 0 
        loop over entries: DO
        
          read(nu_mdl,*,iostat=status) (dum(i), i = 1, ncol)
	  
          if (status /= 0) then
            EXIT loop over entries
          end if
	  
          nentries = nentries + 1
      	  
          depth_in(nentries) = dum(icol_dh)
          vel_in(nentries) = dum(icol_vel)
          if(vel_in(nentries) == 0.0) then      ! check for the last layer
            nentries = nentries - 1      
          EXIT loop over entries
          end if
      
          if (dens_from_model) then
            dens_in(nentries) = dum(icol_dens)
!          write(nu_cabsAmps, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, 
!     :             ' density is from the model file '
          else
            dens_in(nentries) = 0.0
!          write(nu_cabsAmps, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, 
!     :             ' density is from the subroutine '
          end if
      
          if (q_from_model) then
            q_in(nentries) = dum(icol_q)
!          write(nu_cabsAmps, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, 
!     :             ' 1/Q is from the model file '
          else
            q_in(nentries) = 0.0
!          write(nu_cabsAmps, '(a, 1x,i5, a)')
!     :   ' For input model layer ', nentries, ' 1/Q = 0.0 '
          end if
    
        END DO loop over entries
        
        close(unit=nu_mdl)

! Find out something about the type of velocity model:

        if (depth_in(1) == 0.0) then
          write(*,*)
          write(*, '(a)') 
     :  '   Assuming the model in file '//f_mdl(1:nc_f_mdl)//
     :  ' to be PIECEWISE CONTINUOUS;'//
     :    ' if not, stop program using ctl-brk, modify input file,'//
     :    ' and start again. '
          constant_vel = 'N'
        else
          constant_vel = 'Y'
        end if
 
! If constant velocity layers, convert into a velocity-depth function (with
! repeated depths at interfaces) so that it is in the same format as a 
! piecewise continuous function.

! **** NOTE: the depths here are for the model, not the depths corresponding to output frequencies
!      which can be much greater than the model depth ******

        IF (constant_vel == 'Y') THEN ! NOTE: There are nlayers of entries, even if the "depth" is d2b rather than thick
          nlayers = nentries
          idepth = 0
          d2b(0) = 0.0
          DO ilayer = 1, nlayers  
! first fill in depth-to-bottom array:
            if (type_of_depth(1:nc_type_of_depth) == 'D2B') then
              d2b(ilayer) = depth_in(ilayer)
            else if (type_of_depth(1:nc_type_of_depth) == 'THK') then
              d2b(ilayer) = d2b(ilayer-1) + depth_in(ilayer)
            else
              print *,' type_of_depth = '//
     :                type_of_depth(1:nc_type_of_depth)
              print *,' which is not a legal value; QUITTING'
              close(nu_cabsAmps)
              close(nu_sum)
              stop
            end if
          
! now assign array values for the interfaces above and below each layer:
!   first above:
            idepth = idepth + 1
            depth(idepth) = d2b(ilayer-1)
            vel(idepth) = vel_in(ilayer)
            dens(idepth) = dens_in(ilayer)  
          
!  Fill in density from velocity-density relation if density = 0.0

            if (dens(idepth) == 0.0) then  
              if (dens_default == 0.0) then
                dens(idepth) = density(vel(idepth),
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
              else
                dens(idepth) = dens_default
              end if
            end if
          
            if (q_from_model) then
              qi(idepth) = q_in(ilayer)
            else
              qi(idepth) = 0.0
            end if
          
            if (qi(idepth) > 1.0) qi(idepth) = 1.0/qi(idepth) ! cnvrt to 1/q

!   now below:
            idepth = idepth + 1
            depth(idepth) = d2b(ilayer)
            vel(idepth) = vel_in(ilayer)
            dens(idepth) = dens_in(ilayer)           
!  Fill in density from velocity-density relation if density = 0.0
            if (dens(idepth) == 0.0) then  
              if (dens_default == 0.0) then
                dens(idepth) = density(vel(idepth),
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
              else
                dens(idepth) = dens_default
              end if
            end if
            if (q_from_model) then
              qi(idepth) = q_in(ilayer)
            else
              qi(idepth) = 0.0
            end if
            if (qi(idepth) > 1.0) qi(idepth) = 1.0/qi(idepth) ! cnvrt to 1/q
          END DO
          ndepths = idepth

        ELSE                 ! constant_vel /= 'Y'

          ndepths = nentries
          DO idepth = 1, ndepths
            depth(idepth) = depth_in(idepth)
            vel(idepth) = vel_in(idepth)
            dens(idepth) = dens_in(idepth)
!  Fill in density from velocity-density relation if density = 0.0
            if (dens(idepth) == 0.0) then  
              if (dens_default == 0.0) then
                dens(idepth) = density(vel(idepth),
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
              else
                dens(idepth) = dens_default
              end if
            end if
            if (q_from_model) then
              qi(idepth) = q_in(idepth)
            else
              qi(idepth) = 0.0
            end if
            if (qi(idepth) > 1.0) qi(idepth) = 1.0/qi(idepth) ! cnvrt to 1/q
          END DO
        
        END IF

        if (dens_ref_in == 0.0) then
          dens_ref = density(vel_ref,
     :                     specify_dens_coeff,
     :                     dens_low, vel_low,
     :                     dens_high, vel_high, dd_dv, units_c)
        else
          dens_ref = dens_ref_in
        end if
         

! NOW THE MODELS ARE IN THE SAME FORMAT (CONTINUOUS WITH DEPTH, EVEN IF STEPS EXIST)
! AND WE CAN TREAT IT IN THE SAME WAY.


! extrapolate model here, if desired.  Save ndepths_in to be used in saving fmin4model_in. Call construct_constant_velocity_model here
!----------------
!
!Work in progress
!
!----------------
!      if(xtrapmodel) then
!        if (xtraptype=='P') then
!          pxtrap = 
!     :     alog10(vx/vel(ndepths_in))/alog10(zx/depth(ndepths_in))
!        end if
!      
!        DeltaLogZxtrap = (alog10(zx/depth(ndepths))/real(nxtrap-1) 
!        do iz = 1, nxtrap
!          ndepths = ndepths + 1
!          depth(ndepths) = 10.0**
!     :         (real(iz - 1)*DeltaLogZxtrap+depth(ndepths_in))
!          if(xtraptype=='L') then
!       
!     : xtrapmodel, xtraptype, zx, vx, nxtrap, 



!**********
!
! CONSTRUCT THE EQUIVALENT CONSTANT-VELOCITY LAYERED MODEL:
!
!**********


       ndepthsOriginalModel = ndepths
       call construct_constant_velocity_model(
     :   ndepthsOriginalModel,  
     :   depth, vel, dens, qi, 
     :   vel_ref, dens_ref, aoi_src, p,
     :   akappa_fixed,
     :   freqlow4model, freqhigh4model, nfreq4model,
     :   log_spaced_freqs4model, 
     :   d2lyrbot, thick_layer, vel_layer, dens_layer, 
     :   qi_kappa_fixed, qi_layer, 
     :   tt2lyrbot, avgvel2lyrbot, avgdens2lyrbot, avgaoi2lyrbot,
     :   akappa2lyrbot,
     :   freq_SRIamps, amp_SRIamps, akappa_SRIamps, 
     :   ampxkappa_SRIamps, akappa4SRIamps, nCVLlayers,
     :   fmin4OrigModel, MLAYR, MNFREQS )

        
!!**********
!
!  DO COMPUTATIONS
!
!**********

        CALL NrattleAmpComputations(d2lyrbot, thick_layer, 
     :   vel_layer, dens_layer, qi_layer, 
     :   nCVLlayers, nlay_src, vel_ref, dens_ref_in,
     :   aoi_src, zdepth, freq, nfreq_in, CmplxAmp, MLAYR, MNFREQS )


!**********
!
!  WRITE OUTPUT
!
!**********

        write(nu_cabsAmps, '(a)') ' Program nRattleBatch'

        write(nu_cabsAmps,*)

        write(nu_cabsAmps, '(a)') 
     :     ' Output file: '//f_cabsAmps(1:nc_f_cabsAmps)
        write(nu_cabsAmps, '(a)')  
     :  ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
        write(nu_cabsAmps, '(a)')  
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'

        write(nu_cabsAmps,*)
      
        write(nu_cabsAmps, '(2a, 1x,i5)') 
     :    ' Model file: '//f_mdl(1:nc_f_mdl),
     :                             ', for which nentries = ', nentries

        if (dens_from_model) then
          write(nu_cabsAmps, '(a)')
     :   ' density in each layer is from the model file unless the '//
     :    'entry is 0.0 '
        else
          write(nu_cabsAmps, '(a)')
     :   ' density in each layer is from the density subroutine '
        end if

        if (q_from_model) then
          write(nu_cabsAmps, '(a)')
     :   ' 1/Q in each layer is from the model file '
        else
          write(nu_cabsAmps, '(a)')
     :   ' 1/Q in each layer = 0.0 '
        end if

        write(nu_cabsAmps,'(a,i4)') 
     :  ' Layer number of effective halfspace ='  ,nlay_src
        write(nu_cabsAmps,'(a,f7.2)') 
     :  ' Angle of incidence in halfspace ='  ,aoi_src

        write(nu_cabsAmps,*)

        write(nu_cabsAmps,'(a,i4)') 
     :    ' total number of layers in model = ', nCVLlayers

        write(nu_cabsAmps,*)

        write(nu_cabsAmps,90123) vel_ref, dens_ref_in
90123 FORMAT (' Substratum S-velocity =',F10.4/
     *        ' Substratum density =',F10.4)

        write(nu_cabsAmps,90122) nfreq4model, 
     :       freqlow4model, freqhigh4model  
        write(nu_cabsAmps,*)
        write(nu_cabsAmps,'(1x,a,1x,f7.5)') 
     :         'fmin4OrigModel = ', fmin4OrigModel 

        write(nu_cabsAmps,*)

        write(nu_cabsAmps,90120) nfreq_in, freqlow, freqhigh
90120 FORMAT (' nfreq for amps = ',i6/
     :        ' freqlow for amps (Hz) = ',f8.3/
     :        ' freqhigh for amps (Hz) = ',f8.3)
        write(nu_cabsAmps,*)

        tempbuf = ' '

        tempbuf = f_cabsAmps(1:index(f_cabsAmps,'.')-1)
        call trim_c(tempbuf,nchar_tb)
        col1 = ' '
        col1(9-nchar_tb:8) = tempbuf(1:nchar_tb)
        col1(9:12) = '-frq'
        col2 = ' '
        col2(9-nchar_tb:8) = tempbuf(1:nchar_tb)
        col2(9:12) = '-per'
        col3 = ' '
        col3(9-nchar_tb:8) = tempbuf(1:nchar_tb)
        col3(9:12) = '-amp'
        write(nu_cabsAmps, '(1x,a, 1x,a, 1x,a)') col1, col2, col3
 
        cabsAmps loop over frequencies: DO IFRQ=1,NFREQ_IN

          f = freq(ifrq)

          if (f == 0.0) then
            per = 1.0e+06
          else
            per = 1.0/f
          end if
          WRITE (nu_cabsAmps,'(1x,f12.7, 2(1x,es12.4))') f, per, 
     :          cabs(CmplxAmp(ifrq))
         
        END DO cabsAmps loop over frequencies        


      
        close(unit=nu_cabsAmps)
        
        if(writeFileWithModel) then

          write(nu_MdlAMps, '(a)') ' Program nRattleBatch'
          write(nu_MdlAMps, '(a)') 
     :      ' Output file: '//f_MdlAmps(1:nc_f_MdlAMps)
          write(nu_MdlAMps, '(a)') 
     : ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
          write(nu_MdlAMps, '(a)') 
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'
          write(nu_MdlAMps,*)
         
          write(nu_MdlAMps,90122) nfreq4model, 
     :       freqlow4model, freqhigh4model  
90122 FORMAT (' nfreq4model = ',i6/
     :        ' freqlow4model (Hz) = ',f8.3/
     :        ' freqhigh4model (Hz) = ',f8.3)
          write(nu_MdlAMps,*)
          write(nu_MdlAMps,'(1x,a,1x,f7.5)') 
     :         'fmin4OrigModel = ', fmin4OrigModel 
          write(nu_MdlAMps,*)
 
          write(nu_MdlAMps,'(a,f10.4)')
     :    ' Depth at which response is computed = ', zdepth
          write(nu_MdlAMps,*)
          write(nu_MdlAMps,'(a,i4)') 
     :      ' total number of layers in model = ', nCVLlayers
          write(nu_MdlAMps,*)

! Write column headers for model:

          write(nu_MdlAMps,'(
     :      1x,a, 7x,a, 3x,a,
     :      10x,a, 1x,a, 7x,a
     :                                           )' )
     :     'lyr_num', 'd2bot', 'thickness', 
     :      'vs', 'density', '1/q' 

          do i = 1, nCVLlayers
      
            write(nu_MdlAMps,'(
     :                      4x,i4, 1x,es11.4, 1x,es11.4, 
     :                      1x,es11.4, 1x,f7.4, 1x,f9.4
     :                                                 )' ) 
     :          i, d2lyrbot(i), thick_layer(i),
     :          vel_layer(i), dens_layer(i), qi_layer(i)
         
          end do
      
          write(nu_MdlAMps,*)
          write(nu_MdlAMps,'(a,i4)') 
     :      ' Layer number of effective halfspace ='  ,nlay_src
          write(nu_MdlAMps,'(a,f7.2)') 
     :      ' Angle of incidence in halfspace ='  ,aoi_src
          write(nu_MdlAMps,*)

          write (nu_MdlAMps,90123) vel_ref, dens_ref
          write(nu_MdlAMps,*)

          WRITE(nu_MdlAMps,90320)
90320 FORMAT (t4,'nfreq', t14,'real(s)', t26,'imag(s)',
     :        t41,'freq', t50,'cabs(s)' )

          loop over frequencies: DO IFRQ=1,NFREQ_IN
          
            f = freq(ifrq)
	     
            WRITE (nu_MdlAMps,92300) IFRQ,CmplxAmp(ifrq), f, 
     :                        cabs(CmplxAmp(ifrq))
92300    FORMAT (t4,i5, 4(1x,1pe11.4))
          
          END DO loop over frequencies        

          close(unit=nu_MdlAmps)
           
        end if

 

        if (writeSriAmps) then
          write(nu_SriAmps, '(a)') ' Program nRattleBatch'

          write(nu_SriAmps,*)

          write(nu_SriAmps, '(a)') 
     :     ' Output file: '//f_SriAmps(1:nc_f_SriAmps)
          write(nu_SriAmps, '(a)')  
     :  ' Date: '//date_c(1:4)//'-'//date_c(5:6)//'-'//date_c(7:8)
          write(nu_SriAmps, '(a)')  
     : ' Time: '//time_c(1:2)//'h'//time_c(3:4)//'m'//time_c(5:10)//'s'

          write(nu_SriAmps,*)
      
          write(nu_SriAmps, '(2a, 1x,i5)') 
     :    ' Model file: '//f_mdl(1:nc_f_mdl),
     :                             ', for which nentries = ', nentries

          if (dens_from_model) then
            write(nu_SriAmps, '(a)')
     :   ' density in each layer is from the model file unless the '//
     :    'entry is 0.0 '
          else
            write(nu_SriAmps, '(a)')
     :   ' density in each layer is from the density subroutine '
          end if

          if (q_from_model) then
            write(nu_SriAmps, '(a)')
     :   ' 1/Q in each layer is from the model file '
          else
            write(nu_SriAmps, '(a)')
     :   ' 1/Q in each layer = 0.0 '
          end if

          write(nu_SriAmps,'(a,i4)') 
     :  ' Layer number of effective halfspace ='  ,nlay_src
          write(nu_SriAmps,'(a,f7.2)') 
     :  ' Angle of incidence in halfspace ='  ,aoi_src

          write(nu_SriAmps,*)

          write(nu_SriAmps,'(a,i4)') 
     :    ' total number of layers in model = ', nCVLlayers

          write(nu_SriAmps,*)

          write(nu_SriAmps,90123) vel_ref, dens_ref_in
 
          write(nu_SriAmps,90122) nfreq4model, 
     :       freqlow4model, freqhigh4model  
          write(nu_SriAmps,*)
          write(nu_SriAmps,'(1x,a,1x,f7.5)') 
     :         'fmin4OrigModel = ', fmin4OrigModel 

          write(nu_SriAmps,*)

          write(nu_SriAmps,90120) nfreq_in, freqlow, freqhigh

          write(nu_SriAmps,*)

          write(nu_SriAmps, '(9x,a, 10x,a, 7x,a, 8x,a, 10x,a, 7x,a)')  
     :     'freq', 'per', 'SRIamp', 'fnorm', 'eta', 'SRImod'
  
          SriAmps loop over frequencies: DO IFRQ=1,nCVLlayers

            f = freq_SRIamps(ifrq)

            if (f == 0.0) then
              per = 1.0e+06
            else
              per = 1.0/f
            end if

           EtaFromCSV = 
     :         EtaForSRImodFromCSV(f, fmin4OrigModel, 
     :            log10fnorm, eta, neta)

            WRITE (nu_SriAmps,'(1x,f12.7, 6(1x,es12.4))') 
     :          f, per, amp_SRIamps(ifrq), 
     :          f/fmin4OrigModel, 
     :          EtaFromCSV, amp_SRIamps(ifrq)**(2.0*EtaFromCSV)
         
        END DO SriAmps loop over frequencies        

        close(unit=nu_SriAmps)

        end if
        
      
        write(*, '(a)') ' FINISHED --- Loop back for another model'
        write(*,*)
      
      END DO loop over files
 
      close(nu_sum)
      close(nu_ctl)
 
!!DEBUG
!        close(unit=nu_debug)
!!DEBUG

        write(*, '(a)') ' FINISHED PROCESSING ALL FILES'

!      STOP

      END


      include 'site_amp_util_subs.for'


































 