      Program Vel2TT

! Read d, v file and computes travel time (use to compute v30 etc from
! borehole, Rosrine logs).  From the travel time a number of other
! quantities are computed, including the average velocity in each layer,
! the cumulative average velocity, the fqtr frequency for each depth,
! and the square-root impedance amplification.  Various assumptions can be made about
! the densities.

! Here are the output quantities (for each layer):
!  d2b vlayer tt2bot_of_layer vavg2bot_of_layer slayer(m/km) savg2bot_of_layer(sec/km)
!  fqtr vref dens_ref dens_avg amp

! The format of the input model can have any number of header lines,
! as long as they start with "!".  The model values assume
! a column of depth (or thickness) followed by a column with velocity.

! Note that if a PS suspension log, the velocity model starts from the surface,
! either assuming that the first velocity in the model is the velocity of the
! "missing" layer between the surface and the depth of the first value
! (more precisely, the depth from the surface to the depth of the first PS
! value minus half of the depth interval between the first and second PS
! values), or using a velocity from the control file for the velocity
! of the first layer.  For a PS log model, the number of layers equals
! the number of input values plus one.

! Reads info from a control file; here is an example:

!!Control file for vel2tt
!! Revision of program involving a change in the control file on this date:
!   02/12/16
!!Name of summary file:
!   vel2tt.sum
!!Format of input files: 1= svel, 2 = other
!! Note: if svel, then many of the following parameters are not needed,
!! but placeholder entries are still needed.
! 2
!!Are the velocities from a suspension log survey?
!!If yes, assume that the velocity value is at the midpoint of the layer
! N
!!Velocity for first layer (0 = use the first velocity of the input)
!! Use this for sensitivity studies of vavg for PS log models that do not
!! extend to the surface.   
!   0.0
!!Depth to bottom (d2b) or layer thickness (thk) (note: a value of 0.0 for
!!the first depth from the model file will signal that the model is a
!!piecewise continuous model)?
!   thk
!!Lines to skip to reach data (<0, use skipcmnt; 0, skip none; > 0, specify)
!   +1
!!Ncolumns, column numbers for depth or thickness (allows for the first column
!!being an index) and the velocity 
!   2 1 2
!!Enter units of velocity (kps, mps) (depth units are assumed to be consistent
!!with the velocity units---km for kps and m for mps):
!   mps
!!vref for rough amp calculation:
!  3700.0
!!Let depth extend beyond bottom of model to reach 30 m:
! N
!!Depth increment, deepest depth (will be adjusted if the answer to
!! the question above is "N"):
! 0.5 100.0
!!Calculate tt, amps at bottom even if the maximum of the incremented
!! depths are less than this
! Y
!!Character string to append to end of each input file to create
!!the name of the output file (be sure to include the period if wanted):
! .vtt
!!List of input files
!!Make sure that there are no blank entries!
!! thickness
!pfo_composite_velocity_model.mdl
!stop
  
! Format of input files:
! headers (preceded by "!" if more than one)
! depths, vel (see control file, can have other columns)

! Note: add "!" to beginning of headers in input files (because can then
! have an arbitrary number)

! VERY IMPORTANT --- Make sure that there are no blank entries; if there are
! the next nonblank will be read for the value that should have been
! in the blank space (so a velocity will be read as a depth, and vice versa).

! Dates:   
!        08/09/07 - Written by D. Boore, based on vel2tt
!        02/12/11 - abs depth read from input (TNSMP data had negative depths)
!        02/12/16 - Fixed problem that gradient models were not treated correctly.
!                 - This version removes density computation.  If these are needed for
!                   amplification calculations, use site_amp or site_amp_batch instead
!                 - Renamed it back to vel2tt, because the amp calculations should be done
!                   using site_amp or site_amp_batch, which are better implementations of the
!                   square-root-impedance method.
!        06/09/16 - Change "d_xtrap" to "d_xtrap_to_30m"
!        11/02/20 - Change name of column 1 from "!d2b" to "d" (and change other column names accordingly)

      real depth_in(0:10000), d2b_in(0:10000), vel_in(0:10000), 
     :     slope_v(0:10000), depth_temp(0:10000), vel_temp(0:10000),
     :     d2b(0:10000), d4tt(0:10000), tt(0:10000),
     :     depth_model(10000), vel_model(10000), 
     :     tt_model(10000), avgvel(10000), vel(10000)
      real dum(10)
      character f_sum*100, f_in*200, f_tt*200, buf*200, buf4*4, ext*60
      character path*200, f_in_with_path*400
      character cmnts2skip(50)*80, type*1, constant_vel*1, 
     :          type_of_depth*10, units_c*10

      character date_ctl_correct*8, date_ctl_in*30

      integer status, vel_file_format

      character f_ctl*40
      logical f_exist, susp_log, extend_depth, specify_density
      logical compute_for_bottom

      pi = 4.0 * atan(1.0)
      d2r = pi/180.0

      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (Enter = vel2tt.ctl; '//
     :                                 'ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'vel2tt.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/12/16'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong version date; STOP!'
        write(*,'(a)') 
     :     ' The correct date is '//
     :            date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
      
! Get summary file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')
      write(nu_sum,'(9x,a, 8x,a, 6x,a, 6x,a, 10x,a, 1x,a)')
     :   'file', 'd2b', 'thk_max', 'd_xtrap_to_30m', 'v30', 'class'

! DEBUG
      call get_lun(nu_debug)
      open(unit=nu_debug,file='debug.out',status='unknown')
! DEBUG

! What file format?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) vel_file_format
      
! DEBUG
      write(nu_debug,'(a,1x,i5)') ' vel_file_format = ',vel_file_format
! DEBUG
      
! Are velocities from a suspension log?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         susp_log = .true.
      else
         susp_log = .false.
      end if

! Get velocity of first layer:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vel1
      
!Depth to bottom (d2b) or thickness (thk)?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      type_of_depth = ' '
      read(nu_ctl, '(a)') type_of_depth
      call trim_c(type_of_depth, nc_type_of_depth)
      call upstr(type_of_depth)

! Lines to skip (<0 means use skpcmnts; 0 means skip none):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) nskip

! Ncolumns, Column number for depth/thickness and velocity?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) ncol, icol_dh, icol_vel

!Specify units of velocity (kps, mps):
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      units_c = ' '
      read(nu_ctl,'(a)') units_c
      call trim_c(units_c, nc_units_c)
      call upstr(units_c)
      if (units_c(1:3) /= 'MPS' .and. units_c(1:3) /= 'KPS') then
        print *, ' ERROR: units_c(1:3) >= MPS or KPS; it equals '//
     :           units_c(1:3)//' STOP!!!'
        stop
      end if
      
! Get reference velocity:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) vref
      
! Extend depth?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         extend_depth = .true.
      else
         extend_depth = .false.
      end if

! Get ddepth4tt, depth_max4tt:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, *) ddepth4tt, depth_max4tt_in
      
! Compute amps for bottom?
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf, nc_buf)
      call upstr(buf)
      if (buf(1:1) == 'Y') then
         compute_for_bottom = .true.
      else
         compute_for_bottom = .false.
      end if

! Get extension for output file name:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl, '(a)') ext
      call trim_c(ext,nc_ext)
 
! Read and process files:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      loop over files: DO

        buf = ' '
        read(nu_ctl,'(a)',iostat=status) buf
        if(status /= 0) then
          EXIT loop over files
        end if
        call trim_c(buf, nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(nc_buf == 0 .or. buf(1:1) == '!') then
          CYCLE loop over files
        end if
        if(buf4(1:4) == 'STOP') then
          EXIT loop over files
        end if
        
        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)
! DEBUG
      write(nu_debug,'(a)') 
     :   ' f_in = '//f_in(1:nc_f_in)
! DEBUG

        f_exist = .false.
        inquire(file=f_in(1:nc_f_in), exist=f_exist)
        if (.not. f_exist) then
           write(*,'(a)') 
     :  ' ******* FILE '//f_in(1:nc_f_in)//
     :  ' DOES NOT EXIST ******* '
           write(*,'(a)') ' Skipping to next file'
           CYCLE loop over files
        end if
      
        call get_lun(nu_in)
        open(nu_in,file=f_in(1:nc_f_in),
     :     status='unknown')

        if (vel_file_format == 1 ) then  ! svel file format; may want to use CASE later, if allow more formats
          call skip(nu_in, 6)
! Get nlayers:
          buf = ' '
          read(nu_in,'(a)') buf
          call trim_c(buf, nc_buf) 
! Search for first occurrence of "=" after column 1
          ncol_start = 1
          do i = ncol_start, nc_buf
            if(buf(i:i) == '=') then
              ncol_start = i + 1
              exit
            end if
          end do
! Read nlayers:
          read(buf(ncol_start:nc_buf),*) nlay_in
! DEBUG
            write(nu_debug,'(1x,a, 1x, i6)') ' nlay_in = ', nlay_in
! DEBUG
! Get velocity model:
          call skip(nu_in,2)
          d2b_in(0) = 0.0
          do i = 1, nlay_in
            read(nu_in,*) dum1, dum2, dum3, dum4, dum5, dum6, dum7,
     :                depth_in(i), dum8, vel_in(i)
          end do
        
        else   ! not svel format

          if (nskip < 0) then
            call skipcmnt(nu_in, cmnts2skip, nc_cmnts2skip)
          else if (nskip > 0) then
            call skip(nu_in, nskip)
          end if
 
          nlay_in = 0
          depth_in(0) = 0.0
         
          loop over layers: DO
            read(nu_in,*,iostat=status) (dum(i), i = 1, ncol)
         
            if (status /= 0) then
              EXIT loop over layers
            end if
         
            nlay_in = nlay_in + 1
            depth_in(nlay_in) = abs(dum(icol_dh))
            vel_in(nlay_in) = dum(icol_vel)

            write(*,*) depth_in(nlay_in), vel_in(nlay_in)
! DEBUG
            write(nu_debug,*) depth_in(nlay_in), vel_in(nlay_in)
! DEBUG
       
          END DO loop over layers
       
        end if	  ! not svel format
      
        close(nu_in)

! Find out something about the type of velocity model:

        if (depth_in(1) == 0.0) then
          write(*,*)
          write(*, '(a)') 
     :  '   Assuming the model to be PIECEWISE CONTINUOUS;'//
     :    ' if not, stop program using ctl-brk, modify input file,'//
     :    ' and start again. '
          constant_vel = 'N'
        else
          constant_vel = 'Y'
        end if
 

!
!$$$$$$$$$$$$$$$$$  NEEDS WORK !!!!!!!!!! $$$$$$$$$$$$$$$$$$$$$$$$$$
!
! The code below for a suspension log model needs work; I cannot remember what I assume 
! about the model (a typical model is a a sequence of depths and velocities).
!
! If a suspension log, assume that the depth corresponds to the middle of a layer of 
! constant velocity.  Redo the input model as if it is a stack of constant velocity layers.

      IF (susp_log) then

        vel_temp = vel_in
        depth_temp = depth_in
        nlay_temp = nlay_in
        
        if (vel1 /= 0.0) then
          vel_in(1) = vel1
        end if

        depth_in(0) = 0.0
        depth_in(1) = 
     :         depth_temp(1) - 0.5*(depth_temp(2) - depth_temp(1))

        nlay_in = nlay_temp + 1

! Now assume that the depth is the midpoint of each layer
! 

        do i = 1, nlay_temp-1
          depth_in(i+1) = 0.5*(depth_temp(i+1) + depth_temp(i))
          vel_in(i+1) = vel_temp(i)
        end do

! depth of last point is a special case:

        vel_in(nlay_in) = vel_temp(nlay_temp)
        depth_in(nd4tt) = depth_temp(nlay_temp) + 
     :          0.5*(depth_temp(nlay_temp) - depth_temp(nlay_temp-1))

      END IF
!
!$$$$$$$$$$$$$$$$$  NEEDS WORK !!!!!!!!!! $$$$$$$$$$$$$$$$$$$$$$$$$$



! If constant velocity layers, convert into a velocity-depth function (with
! repeated depths at interfaces) so that it is in the same format as a 
! piecewise continuous function.

        IF (constant_vel == 'Y') then
          nlayers = nlay_in
          idepth = 0
          d2b(0) = 0.0
          DO ilayer = 1, nlayers
! first fill in depth-to-bottom array:
            if (type_of_depth(1:nc_type_of_depth) == 'D2B') then
              d2b(ilayer) = depth_in(ilayer)
            else if (type_of_depth(1:nc_type_of_depth) == 'THK') then
              d2b(ilayer) = d2b(ilayer-1) + depth_in(ilayer)
            else
              print *,' type_of_depth = '//
     :                type_of_depth(1:nc_type_of_depth)
              print *,' which is not a legal value; QUITTING'
              close(nu_sum)
              stop
            end if
          
! now assign array values for the interfaces above and below each layer:
!   first above:
            idepth = idepth + 1
            depth_model(idepth) = d2b(ilayer-1)
            vel_model(idepth) = vel_in(ilayer)

!   now below:
            idepth = idepth + 1
            depth_model(idepth) = d2b(ilayer)
            vel_model(idepth) = vel_in(ilayer)
          END DO  ! end loop over layers
          ndepths_model = idepth
        ELSE                 ! constant_vel /= 'Y'
          ndepths_model = nlay_in
          do idepth = 1, ndepths_model
            depth_model(idepth) = depth_in(idepth)
            vel_model(idepth) = vel_in(idepth)
          end do  ! end loop over layers
        END IF

! Now the model is in a standard format and we can treat it in the
! same way, regardless if it a stack of constant velocity layers or a mixed set
! of linear gradients and constant thickness layers.

! Compute max thickness of input as a way of capturing missing data:
!      thk_max = 0.0
      thk_max = depth_model(1)   ! to catch the first point being at some depth
      do i = 2, ndepths_model
        thk = depth_model(i) - depth_model(i-1)
        if (thk > thk_max) thk_max = thk
      end do

!##############################
! Compute travel times at the model depths
        tt_model(1) = 0.0
        avgvel(1) = vel_model(1)
        DO id = 2, ndepths_model
          del_z = depth_model(id)-depth_model(id-1)
          if (vel_model(id) == vel_model(id-1)) then   ! constant velocity
            slope_v(id-1) = 0.0 
            tt_model(id) = tt_model(id-1) + del_z/vel_model(id)
          else
            if(depth_model(id) == depth_model(id-1)) then  ! step change
              slope_v(id-1) = 1.0e10
              tt_model(id) = tt_model(id-1)
            else
              slope_v(id-1) = (vel_model(id)-vel_model(id-1))/ del_z
              tt_model(id) = tt_model(id-1) + 
     :          (1.0/slope_v(id-1))*alog(vel_model(id)/vel_model(id-1))
            end if
          end if
          avgvel(id) = depth_model(id)/tt_model(id)
        END DO
!##############################


! Set up depths at which the times will be calculated:

! For sensitivity studies, allow v in first layer to be different than
! the first tabulated value, with a d2b as computed below.

! Compute ndepths and fill d4tt

        if (depth_max4tt_in <= depth_model(ndepths_model)) then
          depth_max4tt = depth_max4tt_in
        else
          if (extend_depth) then
            depth_max4tt = depth_max4tt_in
          else
            depth_max4tt = depth_model(ndepths_model)  
          end if
        end if
      
        nd4tt = depth_max4tt/ddepth4tt + 0.01 ! for roundoff

! DEBUG
      print *,
     :  'depth_model(ndepths_model), depth_max4tt, ddepth4tt, nd4tt = '
      print *,
     :   depth_model(ndepths_model), depth_max4tt, ddepth4tt, nd4tt
! DEBUG
      
        d4tt(1) = ddepth4tt
        do i = 2, nd4tt
          d4tt(i) = d4tt(i-1) + ddepth4tt
        end do
      
        if ( d4tt(nd4tt) < 
     :          depth_model(ndepths_model) .and. compute_for_bottom )
     :                                then
          nd4tt = nd4tt + 1
          d4tt(nd4tt) = depth_model(ndepths_model)
        end if
 

!############@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ 

      call get_lun(nu_tt)
      f_tt = ' '
      f_tt = f_in(1:nc_f_in)//ext(1:nc_ext)
      call trim_c(f_tt,nc_f_tt)
      
      open(nu_tt,file= f_tt(1:nc_f_tt),status='unknown')
      write(nu_tt,'(a)') '! f_ctl = '//f_ctl(1:nc_f_ctl)
      
      if (vel1 == 0.0) then
        write(nu_tt,'(a)') 
     :       '! vel1 = 0, so vel of first layer from input'
      else
        write(nu_tt,'(a,1x,f7.2)') 
     :       '! vel of first layer specified to be ',vel1
      end if
      
      print *, '! f_in = '//f_in(1:nc_f_in)
      write(nu_tt,'(a)') '! f_in = '//f_in(1:nc_f_in)
      
      print *, ' vel1, vel_model(1), vref = ', vel1, vel_model(1), vref
      
! Compute and write v30
      if (depth_model(ndepths_model) >= 30.0) then
        d_xtrap_to_30m = 0.0
      else
        d_xtrap_to_30m = 30.0 - depth_model(ndepths_model)
      end if
      
      v_halfspace = vref  
      if (units_c(1:3) == 'MPS') then
        d30 = 30.0
      else
        d30 = 0.030
      end if
      tt30 = TT_FOR_PIECEWISE_CONTINUOUS_MODEL(d30, depth_model, 
     :         vel_model, tt_model, ndepths_model, v_halfspace)
      v30 = d30/tt30
      
!      nd_out = 0
!      do i = 1, ndepths
!        if (d4tt(i) <= depth_model(nlay)) then
!           nd_out = nd_out + 1
!        end if
!      end do
        
      write(nu_tt,'(a, 1x,es12.5, 1x,es12.5, 1x,es12.5)') 
     :   '! d_xtrap_to_30m, tt30, v30 = ', d_xtrap_to_30m, tt30, v30
      write(nu_tt,'(2x,a)') f_in(1:nc_f_in)
      write(nu_tt,'(a)') '! nd4tt, dmax, d_xtrap_to_30m, v30, vref:'
      write(nu_tt,
     :  '(2x,i4, 1x,es12.5, 1x,es12.5, 1x,es12.5, 1x,es12.5)')
     :   nd4tt, depth_model(ndepths_model), d_xtrap_to_30m, v30, vref
     
! Compute and write tt and velocities for specified layers:     
      write(nu_tt,'(12x,a, 7x,a, 9x,a, 7x,a, 
     :              1x,a, 3x,a,
     :              9x,a, 4x,a, 10x,a)') 
     :   'd', 'vlayer', 'tt2d', 'vavg2d',
     :   'slayer(ms/m)', 'savg(ms/m)',
     :   'fqtr', 'sqrt_vavg', 'amp'

      tt(0) = 0.0
      d4tt(0) = 0.0

      v_halfspace = vref  

      do i = 1, nd4tt

!        if (d4tt(i) > d2b(nlay)) then
!          close(nu_tt)
!          go to 1000
!        end if
        
        tt(i) = TT_FOR_PIECEWISE_CONTINUOUS_MODEL(d4tt(i), depth_model, 
     :         vel_model, tt_model, ndepths_model, v_halfspace)

        vlayer = (d4tt(i) - d4tt(i-1))/(tt(i) - tt(i-1))
        slayer = 1000.0/vlayer
        vavg = d4tt(i)/tt(i)
        savg = 1000.0/vavg
        fqtr = 1.0/(4.0*tt(i))
        sqrt_vavg = sqrt(vavg)
        amp = sqrt(vref/vavg)
        
        write(nu_tt,'(9(1x,es12.5))') 
     :                 d4tt(i),
     :                 vlayer,
     :                 tt(i), vavg, 
     :                 slayer, savg,
     :                 fqtr, sqrt_vavg, amp
     
      end do

      close(nu_tt)

      call nhrpclss(v30, type)
      write(nu_sum, 
     :  '(1x,a12, 1x,es12.5, 1x,es12.5, 1x,es12.5, 1x,es12.5, 5x,a1)')
     :  f_in(1:nc_f_in), depth_model(ndepths_model), 
     :  thk_max, d_xtrap_to_30m, v30, type

      END DO loop over files
 
      close(nu_sum)
      close(nu_ctl)
 
      end

      include '\forprogs\vel2tt_subroutines.for'
      
      include '\forprogs\tt4vert.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\density.for'
      include '\svel\forprogs\nhrpclss.for'
      include '\forprogs\locate.for'
      include '\forprogs\tt_for_piecewise_continuous_model.for'

      